@@ -35,7 +35,7 @@ from matplotlib import patheffects
 from mslib.msui.mss_qt import get_open_filenames, get_save_filename
 from mslib.msui.mss_qt import ui_kmloverlay_dockwidget as ui
 from PyQt5 import QtGui, QtWidgets, QtCore
-from mslib.utils import save_settings_qsettings, load_settings_qsettings
+from mslib.utils.config import save_settings_qsettings, load_settings_qsettings
 
 
 class KMLPatch(object):
@@ -52,12 +52,9 @@ class KMLPatch(object):
         self.draw()
 
     def compute_xy(self, geometry):
-        lons = []
-        lats = []
-        for coordinates in geometry.coords:
-            lons.append(coordinates[0])
-            lats.append(coordinates[1])
-        return self.map(lons, lats)
+        unzipped = list(zip(*geometry.coords))
+        x, y = self.map.gcpoints_path(unzipped[0], unzipped[1])
+        return x, y
 
     def add_point(self, point, style, name):
         """
@@ -203,7 +200,7 @@ class KMLPatch(object):
 
     def parse_styles(self, kml_doc):
         # exterior_style : <Style> OUTSIDE placemarks
-        # interior style : within <Style>
+        # interior_style : within <Style>
         for exterior_style in kml_doc.styles():
             if isinstance(exterior_style, styles.Style):
                 name = exterior_style.id
@@ -236,7 +233,8 @@ class KMLPatch(object):
         return local_styles
 
     def draw(self):
-        """Do the actual plotting of the patch.
+        """
+        Do the actual plotting of the patch.
         """
         # Plot satellite track.
         self.styles = {}
@@ -248,18 +246,21 @@ class KMLPatch(object):
         self.map.ax.figure.canvas.draw()
 
     def update(self, color=None, linewidth=None):
-        """Removes the current plot of the patch and redraws the patch.
-           This is necessary, for instance, when the map projection and/or
-           extent has been changed.
+        """
+        Removes the current plot of the patch and redraws the patch.
+        This is necessary, for instance, when the map projection and/or
+        extent has been changed.
         """
         if color is not None:
             self.color = color
         if linewidth is not None:
             self.linewidth = linewidth
+        self.remove()
         self.draw()
 
     def remove(self):
-        """Remove this satellite patch from the map canvas.
+        """
+        Remove this satellite patch from the map canvas.
         """
         for patch in self.patches:
             for element in patch:
@@ -279,7 +280,6 @@ class KMLOverlayControlWidget(QtWidgets.QWidget, ui.Ui_KMLOverlayDockWidget):
         self.setupUi(self)
         self.view = view  # canvas
         self.kml = None
-        self.patch = None  # patch refers to plottings on the map
         self.dict_files = {}  # Dictionary of files added; key : patch , color , linewidth
         self.colour = None
         # Connect slots and signals.
@@ -306,31 +306,39 @@ class KMLOverlayControlWidget(QtWidgets.QWidget, ui.Ui_KMLOverlayDockWidget):
 
         self.settings_tag = "kmldock"
         settings = load_settings_qsettings(
-            self.settings_tag, {"filename": "", "linewidth": 5, "colour": (0, 0, 0, 1),
+            self.settings_tag, {"filename": "", "linewidth": 2, "colour": (0, 0, 0, 1),
                                 "saved_files": {}})  # initial settings
 
         self.directory_location = settings["filename"]
+        if parent is not None:
+            parent.viewCloses.connect(self.save_settings)
 
         # Restore previously opened files
         if settings["saved_files"] is not None:
             delete_files = []  # list to store non-existing files
             self.dict_files = settings["saved_files"]
-            for file in self.dict_files:
-                if os.path.isfile(file) is True:
-                    self.create_list_item(file)
+            for fn in self.dict_files:
+                if os.path.isfile(fn) is True:
+                    self.create_list_item(fn)
                 else:
-                    delete_files.append(file)  # add non-existent files to list
-                    logging.info(file + " does not exist in the directory anymore")
-            for file in delete_files:
-                del self.dict_files[file]  # remove non-existent files from dictionary
+                    delete_files.append(fn)  # add non-existent files to list
+                    logging.info("'%s' does not exist in the directory anymore.", fn)
+            for fn in delete_files:
+                del self.dict_files[fn]  # remove non-existent files from dictionary
             self.load_file()
 
-        # When KMLoverlaywidget is opened,it ensures that the color of individual KML files are already shown as icons.
+        # When KMLoverlaywidget is opened, it ensures that the
+        # color of individual KML files are already shown as icons.
         self.set_color_icons()
+        self.view.plot_kml(self)
 
-    def __del__(self):  # destructor
-        for x in self.dict_files:
-            self.dict_files[x]["patch"] = None  # patch object has to be deleted
+    def update(self):
+        for entry in self.dict_files.values():
+            entry["patch"].update()
+
+    def save_settings(self):
+        for entry in self.dict_files.values():
+            entry["patch"] = None  # patch object has to be deleted
         settings = {
             "filename": str(self.directory_location),
             "linewidth": self.dsbx_linewidth.value(),
@@ -347,64 +355,64 @@ class KMLOverlayControlWidget(QtWidgets.QWidget, ui.Ui_KMLOverlayDockWidget):
         if self.listWidget.count() != 0:
             for index in range(self.listWidget.count()):
                 if self.listWidget.item(index).text() in self.dict_files:
-                    file = self.listWidget.item(index).text()
-                    clr = self.dict_files[file]["color"]
-                    self.listWidget.item(index).setIcon(self.show_color_icon(file, clr))
+                    filename = self.listWidget.item(index).text()
+                    clr = self.dict_files[filename]["color"]
+                    self.listWidget.item(index).setIcon(self.show_color_icon(filename, clr))
 
     def select_color(self):
         """
         Sets the color of the selected KML file when Change Colour button is clicked.
         """
         if self.listWidget.currentItem() is not None:
-            file = self.listWidget.currentItem().text()
-            clr = self.set_color(file)
+            filename = self.listWidget.currentItem().text()
+            clr = self.set_color(filename)
             colour = QtGui.QColor(clr[0] * 255, clr[1] * 255, clr[2] * 255)
             self.colour = QtWidgets.QColorDialog.getColor(colour)
-            if self.colour.isValid() and file in self.dict_files:
-                self.dict_files[file]["color"] = self.colour.getRgbF()
+            if self.colour.isValid() and filename in self.dict_files:
+                self.dict_files[filename]["color"] = self.colour.getRgbF()
                 self.flag2 = 1  # sets flag of 0 to 1 when the color changes
-                self.listWidget.currentItem().setIcon(self.show_color_icon(file, self.set_color(file)))
-                self.dict_files[file]["patch"].update(self.dict_files[file]["color"],
-                                                      self.dict_files[file]["linewidth"])
+                self.listWidget.currentItem().setIcon(self.show_color_icon(filename, self.set_color(filename)))
+                self.dict_files[filename]["patch"].update(
+                    self.dict_files[filename]["color"], self.dict_files[filename]["linewidth"])
                 if self.listWidget.currentItem().checkState() == QtCore.Qt.Unchecked:
                     self.flag2 = 1  # again sets to 1 because itemChanged signal due to set icon had changed it to 0
-                    self.checklistitem(file)
+                    self.checklistitem(filename)
 
-    def set_color(self, file):
+    def set_color(self, filename):
         """
         Returns the respective colour of a given file
         """
-        return self.dict_files[file]["color"]
+        return self.dict_files[filename]["color"]
 
     def select_linewidth(self):
         """
         When new value is entered in dsbx_linewidth, it assigns new linewidth to KML file
         """
         if self.listWidget.currentItem() is not None:
-            file = self.listWidget.currentItem().text()
-            if self.dict_files[file]["linewidth"] != self.dsbx_linewidth.value() and file in self.dict_files:
-                self.dict_files[file]["linewidth"] = self.dsbx_linewidth.value()
+            filename = self.listWidget.currentItem().text()
+            if self.dict_files[filename]["linewidth"] != self.dsbx_linewidth.value() and filename in self.dict_files:
+                self.dict_files[filename]["linewidth"] = self.dsbx_linewidth.value()
 
                 self.flag = 1  # sets flag of 0 to 1 when the linewidth changes
-                self.listWidget.currentItem().setIcon(self.show_color_icon(file, self.set_color(file)))
+                self.listWidget.currentItem().setIcon(self.show_color_icon(filename, self.set_color(filename)))
                 # removes and updates patches in the map according to new linewidth
-                self.dict_files[file]["patch"].remove()
-                self.dict_files[file]["patch"].update(self.dict_files[file]["color"],
-                                                      self.dict_files[file]["linewidth"])
+                self.dict_files[filename]["patch"].remove()
+                self.dict_files[filename]["patch"].update(
+                    self.dict_files[filename]["color"], self.dict_files[filename]["linewidth"])
                 if self.listWidget.currentItem().checkState() == QtCore.Qt.Unchecked:
                     self.flag = 1   # again sets to 1 because itemChanged signal due to set icon had changed it to 0
-                    self.checklistitem(file)
+                    self.checklistitem(filename)
 
-    def set_linewidth(self, file):
+    def set_linewidth(self, filename):
         """
         Returns the respective linewidth of a given file
         """
-        return self.dict_files[file]["linewidth"]
+        return self.dict_files[filename]["linewidth"]
 
     def flagop(self):
         """
-        Flag operation method to control the call of self.load_file() whenever th item in the listWidget changes
-        by any means. If select_linewidth and select_color are already executed, then it doesnot calls self.load_file
+        Flag operation method to control the call of self.load_file() whenever the item in the listWidget changes
+        by any means. If select_linewidth and select_color are already executed, then it doesn't calls self.load_file
         otherwise, it always calls when item is changed anywhere.
         """
         if self.flag == 1:
@@ -414,12 +422,12 @@ class KMLOverlayControlWidget(QtWidgets.QWidget, ui.Ui_KMLOverlayDockWidget):
         else:
             self.load_file()    # important for updating patches on map when checkState changes
 
-    def show_color_icon(self, file, clr):
+    def show_color_icon(self, filename, clr):
         """
         Creating object of QPixmap for displaying icon inside the listWidget
         """
-        if self.set_linewidth(file) >= 1:
-            pixmap = QtGui.QPixmap(20, (2 * int(self.set_linewidth(file))))
+        if self.set_linewidth(filename) >= 1:
+            pixmap = QtGui.QPixmap(20, (2 * int(self.set_linewidth(filename))))
         else:
             pixmap = QtGui.QPixmap(20, 1)
         pixmap.fill(QtGui.QColor(clr[0] * 255, clr[1] * 255, clr[2] * 255))
@@ -432,21 +440,21 @@ class KMLOverlayControlWidget(QtWidgets.QWidget, ui.Ui_KMLOverlayDockWidget):
         if self.listWidget.currentItem() is not None:
             # shows the linewidth and color options once any item is clicked.
             self.frame.show()
-            file = self.listWidget.currentItem().text()
-            if file in self.dict_files:
-                self.dsbx_linewidth.setValue(self.set_linewidth(file))
+            filename = self.listWidget.currentItem().text()
+            if filename in self.dict_files:
+                self.dsbx_linewidth.setValue(self.set_linewidth(filename))
 
-    def checklistitem(self, file):
+    def checklistitem(self, filename):
         """
         Checks the file item in ListWidget
         """
-        item_list = self.listWidget.findItems(file, QtCore.Qt.MatchExactly)
-        for item in item_list:
+        for item in self.listWidget.findItems(filename, QtCore.Qt.MatchExactly):
             index = self.listWidget.row(item)
             self.listWidget.item(index).setCheckState(QtCore.Qt.Checked)
 
     def get_file(self):
-        """Slot that opens a file dialog to choose a kml file or multiple files simultaneously
+        """
+        Slot that opens a file dialog to choose a kml file or multiple files simultaneously
         """
         filenames = get_open_filenames(
             self, "Open KML File", os.path.dirname(str(self.directory_location)), "KML Files (*.kml)")
@@ -462,7 +470,8 @@ class KMLOverlayControlWidget(QtWidgets.QWidget, ui.Ui_KMLOverlayDockWidget):
                     self.listWidget.item(index).setIcon(self.show_color_icon(filename, self.set_color(filename)))
 
     def select_file(self, filenames):
-        """Initializes selected file/ files
+        """
+        Initializes selected file/ files
         """
         for filename in filenames:
             if filename is None:
@@ -473,7 +482,7 @@ class KMLOverlayControlWidget(QtWidgets.QWidget, ui.Ui_KMLOverlayDockWidget):
                 self.dict_files[text] = {}
                 self.dict_files[text]["patch"] = None
                 self.dict_files[text]["color"] = (0, 0, 0, 1)
-                self.dict_files[text]["linewidth"] = 5.0  # Sets linewidth to 5.0 if we add new/previously added files.
+                self.dict_files[text]["linewidth"] = 2.0  # Sets linewidth to 2.0 if we add new/previously added files.
 
                 self.directory_location = text  # Saves location of directory to open
                 self.create_list_item(text)
@@ -518,7 +527,6 @@ class KMLOverlayControlWidget(QtWidgets.QWidget, ui.Ui_KMLOverlayDockWidget):
                 self.remove_file()  # recursively since count of for loop changes every iteration due to del of items))
         self.labelStatusBar.setText("Status: KML Files removed")
         if self.listWidget.count() == 0:  # implies no files in ListWidget
-            self.patch = None
             self.dsbx_linewidth.setValue(0.1)  # Shows 0.1 for Linewidth in absence of any file
             self.frame.hide()   # again hide the color and linewidth options when all files are removed.
         # self.load_file() # not sure to keep this or not, works either ways
@@ -528,10 +536,9 @@ class KMLOverlayControlWidget(QtWidgets.QWidget, ui.Ui_KMLOverlayDockWidget):
         Loads multiple KML Files simultaneously and constructs the
         corresponding patches.
         """
-        if self.patch is not None:  # --> self.patch has been initialized before
-            for filename in self.dict_files:  # removes all patches from map, but not from dict_files
-                if self.dict_files[filename]["patch"] is not None:  # since newly initialized files will have patch:None
-                    self.dict_files[filename]["patch"].remove()
+        for entry in self.dict_files.values():  # removes all patches from map, but not from dict_files
+            if entry["patch"] is not None:  # since newly initialized files will have patch:None
+                entry["patch"].remove()
 
         for index in range(self.listWidget.count()):
             if hasattr(self.listWidget.item(index), "checkState") and (
@@ -544,15 +551,14 @@ class KMLOverlayControlWidget(QtWidgets.QWidget, ui.Ui_KMLOverlayDockWidget):
                         self.kml.from_string(kmlf.read().encode('utf-8'))
                         if self.listWidget.item(index).text() in self.dict_files:  # just a precautionary check
                             if self.dict_files[self.listWidget.item(index).text()]["patch"] is not None:  # added before
-                                self.patch = KMLPatch(self.view.map, self.kml,
-                                                      self.set_color(self.listWidget.item(index).text()),
-                                                      self.set_linewidth(self.listWidget.item(index).text()))
-
+                                patch = KMLPatch(self.view.map, self.kml,
+                                                 self.set_color(self.listWidget.item(index).text()),
+                                                 self.set_linewidth(self.listWidget.item(index).text()))
                             else:  # if new file is being added
-                                self.patch = KMLPatch(self.view.map, self.kml,
-                                                      self.dict_files[self.listWidget.item(index).text()]["color"],
-                                                      self.dict_files[self.listWidget.item(index).text()]["linewidth"])
-                            self.dict_files[self.listWidget.item(index).text()]["patch"] = self.patch
+                                patch = KMLPatch(self.view.map, self.kml,
+                                                 self.dict_files[self.listWidget.item(index).text()]["color"],
+                                                 self.dict_files[self.listWidget.item(index).text()]["linewidth"])
+                            self.dict_files[self.listWidget.item(index).text()]["patch"] = patch
 
                 except (IOError, TypeError, et.XMLSyntaxError, et.XMLSchemaError, et.XMLSchemaParseError,
                         et.XMLSchemaValidateError) as ex:  # catches KML Syntax Errors
