@@ -33,10 +33,14 @@
 """
 
 import logging
+import copy
 import numpy as np
+from shapely.geometry import Polygon
 import matplotlib
+from matplotlib.cm import get_cmap
 import matplotlib.path as mpath
 import matplotlib.patches as mpatches
+from matplotlib.collections import PolyCollection
 import mpl_toolkits.basemap as basemap
 try:
     import mpl_toolkits.basemap.pyproj as pyproj
@@ -45,6 +49,12 @@ except ImportError:
     import pyproj
 
 from mslib.msui import mpl_pathinteractor as mpl_pi
+from mslib.utils.airdata import get_airports, get_airspaces
+
+
+OPENAIP_NOTICE = "Airspace data used comes from openAIP.\n" \
+                 "Visit openAIP.net and contribute to better aviation data, free for everyone to use and share."
+OURAIRPORTS_NOTICE = "Airports provided by OurAirports."
 
 
 class MapCanvas(basemap.Basemap):
@@ -53,7 +63,7 @@ class MapCanvas(basemap.Basemap):
     automatically draw a graticule and to redraw specific map elements.
     """
 
-    def __init__(self, identifier=None, CRS=None, BBOX_UNITS=None, PROJECT_NAME=None,
+    def __init__(self, identifier=None, CRS=None, BBOX_UNITS=None, OPERATION_NAME=None,
                  appearance=None, **kwargs):
         """
         New constructor automatically adds coastlines, continents, and
@@ -64,7 +74,7 @@ class MapCanvas(basemap.Basemap):
         Additional arguments:
         CRS -- string describing the coordinate reference system of the map.
         BBOX_UNITS -- string describing the units of the map coordinates.
-        PROJECT_NAME -- string with project name
+        OPERATION_NAME -- string with operation name
 
         """
         # Coordinate reference system identifier and coordinate system units.
@@ -74,8 +84,8 @@ class MapCanvas(basemap.Basemap):
         else:
             self.bbox_units = getattr(self, "bbox_units", None)
 
-        self.project_name = PROJECT_NAME if PROJECT_NAME is not None else self.project_name \
-            if hasattr(self, "project_name") else None
+        self.operation_name = OPERATION_NAME if OPERATION_NAME is not None else self.operation_name \
+            if hasattr(self, "operation_name") else None
 
         # Dictionary containing map appearance settings.
         if appearance is not None:
@@ -91,7 +101,6 @@ class MapCanvas(basemap.Basemap):
                               "colour_land": ((204 / 255.), (153 / 255.), (102 / 255.), (255 / 255.))}
         default_appearance.update(param_appearance)
         self.appearance = default_appearance
-
         # Identifier of this map canvas (used to query data structures that
         # are observed by different views).
         if identifier is not None:
@@ -105,7 +114,10 @@ class MapCanvas(basemap.Basemap):
         # delete the attribute (mr, 08Feb2013).
         if hasattr(self, "epsg"):
             del self.epsg
-        super(MapCanvas, self).__init__(**kwargs)
+        super().__init__(**kwargs)
+
+        self.gc = pyproj.Geod(a=self.rmajor, b=self.rminor)
+
         self.kwargs = kwargs
 
         # Set up the map appearance.
@@ -135,8 +147,8 @@ class MapCanvas(basemap.Basemap):
 
         # Print project name and CRS identifier into figure.
         crs_text = ""
-        if self.project_name is not None:
-            crs_text += self.project_name
+        if self.operation_name is not None:
+            crs_text += self.operation_name
         if self.crs is not None:
             if len(crs_text) > 0:
                 crs_text += "\n"
@@ -147,16 +159,20 @@ class MapCanvas(basemap.Basemap):
             self.crs_text = self.ax.figure.text(0, 0, crs_text)
 
         if self.appearance["draw_graticule"]:
-            try:
-                self._draw_auto_graticule()
-            except Exception as ex:
-                logging.error("ERROR: cannot plot graticule (message: %s - '%s')", type(ex), ex)
+            pass
+            # self._draw_auto_graticule() ; It's already called in mpl_qtwidget.py in MplTopviewCanvas init_map().
         else:
             self.map_parallels = None
             self.map_meridians = None
 
         # self.warpimage() # disable fillcontinents when loading bluemarble
         self.ax.set_autoscale_on(False)
+        if not hasattr(self, "airports") or not self.airports:
+            self.airports = None
+            self.airtext = None
+        if not hasattr(self, "airspaces") or not self.airspaces:
+            self.airspaces = None
+            self.airspacetext = None
 
     def set_identifier(self, identifier):
         self.identifier = identifier
@@ -179,7 +195,7 @@ class MapCanvas(basemap.Basemap):
         super(MapCanvas, self).set_axes_limits(ax=ax)
         matplotlib.interactive(intact)
 
-    def _draw_auto_graticule(self):
+    def _draw_auto_graticule(self, font_size=None):
         """
         Draw an automatically spaced graticule on the map.
         """
@@ -271,8 +287,8 @@ class MapCanvas(basemap.Basemap):
         spacingValues = [0.05, 0.1, 0.25, 0.5, 1, 2, 5, 10, 20, 30, 40]
         deltaLon = mapLonStop - mapLonStart
         deltaLat = mapLatStop - mapLatStart
-        spacingLon = [i for i in spacingValues if i > (deltaLon / 11.)][0]
-        spacingLat = [i for i in spacingValues if i > (deltaLat / 11.)][0]
+        spacingLon = ([i for i in spacingValues if i > (deltaLon / 11.)] + [60])[0]
+        spacingLat = ([i for i in spacingValues if i > (deltaLat / 11.)] + [60])[0]
 
         #   c) parallels and meridians start at the first value in the
         #      spacingLon/Lat grid that's smaller than the lon/lat of the
@@ -283,14 +299,14 @@ class MapCanvas(basemap.Basemap):
         latStart = np.floor((mapLatStart / spacingLat)) * spacingLat
         latStop = np.ceil((mapLatStop / spacingLat)) * spacingLat
 
-        #   d) call the basemap methods to draw the lines in the determined
-        #      range.
+        # d) call the basemap methods to draw the lines in the determined
+        #    range.
         self.map_parallels = self.drawparallels(np.arange(latStart, latStop,
                                                           spacingLat),
-                                                labels=[1, 1, 0, 0], zorder=3)
+                                                labels=[1, 1, 0, 0], fontsize=font_size, zorder=3)
         self.map_meridians = self.drawmeridians(np.arange(lonStart, lonStop,
                                                           spacingLon),
-                                                labels=[0, 0, 0, 1], zorder=3)
+                                                labels=[0, 0, 0, 1], fontsize=font_size, zorder=3)
 
     def set_graticule_visible(self, visible=True):
         """
@@ -328,6 +344,155 @@ class MapCanvas(basemap.Basemap):
             # Update the figure canvas.
             self.ax.figure.canvas.draw()
 
+    def set_draw_airports(self, value, port_type=["small_airport"], reload=True):
+        """
+        Sets airports to visible or not visible
+        """
+        if (reload or not value) and self.airports:
+            if OURAIRPORTS_NOTICE in self.crs_text.get_text():
+                self.crs_text.set_text(self.crs_text.get_text().replace(f"{OURAIRPORTS_NOTICE}\n", ""))
+            self.airports.remove()
+            self.airtext.remove()
+            self.airports = None
+            self.airtext = None
+            self.ax.figure.canvas.mpl_disconnect(self.airports_event)
+        if value:
+            self.draw_airports(port_type)
+
+    def set_draw_airspaces(self, value, airspaces=[], range_km=None, reload=True):
+        """
+        Sets airspaces to visible or not visible
+        """
+        if (reload or not value) and self.airspaces:
+            if OPENAIP_NOTICE in self.crs_text.get_text():
+                self.crs_text.set_text(self.crs_text.get_text().replace(f"{OPENAIP_NOTICE}\n", ""))
+            self.airspaces.remove()
+            self.airspacetext.remove()
+            self.airspaces = None
+            self.airspacetext = None
+            self.ax.figure.canvas.mpl_disconnect(self.airspace_event)
+        if value:
+            country_codes = [airspace.split(" ")[-1] for airspace in airspaces]
+            self.draw_airspaces(country_codes, range_km)
+
+    def draw_airspaces(self, countries=[], range_km=None):
+        """
+        Load and draw airspace data
+        """
+        if not self.airspaces:
+            airspaces = copy.deepcopy(get_airspaces(countries))
+            if not airspaces:
+                logging.error("Tried to draw airspaces without .aip files.")
+                return
+
+            for i, airspace in enumerate(airspaces):
+                airspaces[i]["polygon"] = list(zip(*self.projtran(*list(zip(*airspace["polygon"])))))
+            map_polygon = Polygon([(self.llcrnrx, self.llcrnry), (self.urcrnrx, self.llcrnry),
+                                  (self.urcrnrx, self.urcrnry), (self.llcrnrx, self.urcrnry)])
+            airspaces = [airspace for airspace in airspaces if
+                         (not range_km or range_km[0] <= airspace["bottom"] <= range_km[1]) and
+                         Polygon(airspace["polygon"]).intersects(map_polygon)]
+            if not airspaces:
+                return
+
+            if OPENAIP_NOTICE not in self.crs_text.get_text():
+                self.crs_text.set_text(f"{OPENAIP_NOTICE}\n" + self.crs_text.get_text())
+
+            airspaces.sort(key=lambda x: (x["bottom"], x["top"] - x["bottom"]))
+            max_height = max(airspaces[-1]["bottom"], 0.001)
+            cmap = get_cmap("Blues")
+            airspace_colors = [cmap(1 - airspaces[i]["bottom"] / max_height) for i in range(len(airspaces))]
+
+            collection = PolyCollection([airspace["polygon"] for airspace in airspaces], alpha=0.5, edgecolor="black",
+                                        zorder=5, facecolors=airspace_colors)
+            collection.set_pickradius(0)
+            self.airspaces = self.ax.add_collection(collection)
+            self.airspacetext = self.ax.annotate(airspaces[0]["name"], xy=airspaces[0]["polygon"][0], xycoords="data",
+                                                 bbox={"boxstyle": "round", "facecolor": "w",
+                                                       "edgecolor": "0.5", "alpha": 0.9}, zorder=7)
+            self.airspacetext.set_visible(False)
+
+            def update_text(index, xydata):
+                self.airspacetext.xy = xydata
+                self.airspacetext.set_position(xydata)
+                self.airspacetext.set_text("\n".join([f"{airspaces[i]['name']}, {airspaces[i]['bottom']} - "
+                                                     f"{airspaces[i]['top']}km" for i in index["ind"]]))
+                highlight_cmap = get_cmap("YlGn")
+                for i in index["ind"]:
+                    airspace_colors[i] = highlight_cmap(1 - airspaces[i]["bottom"] / max_height)
+                self.airspaces.set_facecolor(airspace_colors)
+                for i in index["ind"]:
+                    airspace_colors[i] = cmap(1 - airspaces[i]["bottom"] / max_height)
+
+            def on_move(event):
+                if self.airspaces and event.inaxes == self.ax:
+                    cont, ind = self.airspaces.contains(event)
+                    if cont:
+                        update_text(ind, (event.xdata, event.ydata))
+                        self.airspacetext.set_visible(True)
+                        self.ax.figure.canvas.draw_idle()
+                    elif self.airspacetext.get_visible():
+                        self.airspacetext.set_visible(False)
+                        self.airspaces.set_facecolor(airspace_colors)
+                        self.ax.figure.canvas.draw_idle()
+
+            self.airspace_event = self.ax.figure.canvas.mpl_connect('motion_notify_event', on_move)
+
+    def draw_airports(self, port_type):
+        """
+        Load and draw airports and their respective name on hover
+        """
+        if not self.airports:
+            airports = get_airports()
+            if not airports:
+                logging.error("Tried to draw airports but none were found. Try redownloading.")
+                return
+
+            lons, lats = self.projtran(*zip(*[(float(airport["longitude_deg"]),
+                                            float(airport["latitude_deg"])) for airport in airports]))
+            for i, airport in enumerate(airports):
+                airports[i]["longitude_deg"] = lons[i]
+                airports[i]["latitude_deg"] = lats[i]
+
+            airports = [airport for airport in airports if airport["type"] in port_type and
+                        self.llcrnrx <= float(airport["longitude_deg"]) <= self.urcrnrx and
+                        self.llcrnry <= float(airport["latitude_deg"]) <= self.urcrnry]
+            lons = [float(airport["longitude_deg"]) for airport in airports]
+            lats = [float(airport["latitude_deg"]) for airport in airports]
+            annotations = [airport["name"] for airport in airports]
+            if not airports:
+                return
+
+            if OURAIRPORTS_NOTICE not in self.crs_text.get_text():
+                self.crs_text.set_text(f"{OURAIRPORTS_NOTICE}\n" + self.crs_text.get_text())
+
+            self.airports = self.ax.scatter(lons, lats, marker="o", color="r", linewidth=1, s=9, edgecolor="black",
+                                            zorder=6)
+            self.airports.set_pickradius(1)
+            self.airtext = self.ax.annotate(annotations[0], xy=(lons[0], lats[0]), xycoords="data",
+                                            bbox={"boxstyle": "round", "facecolor": "w",
+                                                  "edgecolor": "0.5", "alpha": 0.9}, zorder=8)
+            self.airtext.set_visible(False)
+
+            def update_text(index):
+                pos = self.airports.get_offsets()[index["ind"][0]]
+                self.airtext.xy = pos
+                self.airtext.set_position(pos)
+                self.airtext.set_text("\n".join([annotations[i] for i in index["ind"]]))
+
+            def on_move(event):
+                if self.airports and event.inaxes == self.ax:
+                    cont, ind = self.airports.contains(event)
+                    if cont:
+                        update_text(ind)
+                        self.airtext.set_visible(True)
+                        self.ax.figure.canvas.draw_idle()
+                    elif self.airtext.get_visible():
+                        self.airtext.set_visible(False)
+                        self.ax.figure.canvas.draw_idle()
+
+            self.airports_event = self.ax.figure.canvas.mpl_connect('motion_notify_event', on_move)
+
     def set_fillcontinents_visible(self, visible=True, land_color=None,
                                    lake_color=None):
         """
@@ -517,23 +682,22 @@ class MapCanvas(basemap.Basemap):
         self.ax.figure.canvas.draw()
         return self.image
 
-    def gcpoints2(self, lon1, lat1, lon2, lat2, del_s=100., map_coords=True):
+    def gcpoints2(self, lon0, lat0, lon1, lat1, del_s=100., map_coords=True):
         """
         The same as basemap.gcpoints(), but takes a distance interval del_s
         to space the points instead of a number of points.
         """
         # use great circle formula for a perfect sphere.
-        gc = pyproj.Geod(a=self.rmajor, b=self.rminor)
-        az12, az21, dist = gc.inv(lon1, lat1, lon2, lat2)
+        _, _, dist = self.gc.inv(lon0, lat0, lon1, lat1)
         npoints = int((dist + 0.5 * 1000. * del_s) / (1000. * del_s))
-        lonlats = gc.npts(lon1, lat1, lon2, lat2, npoints)
-        lons = [lon1]
-        lats = [lat1]
+        lonlats = self.gc.npts(lon0, lat0, lon1, lat1, npoints)
+        lons = [lon0]
+        lats = [lat0]
         for lon, lat in lonlats:
             lons.append(lon)
             lats.append(lat)
-        lons.append(lon2)
-        lats.append(lat2)
+        lons.append(lon1)
+        lats.append(lat1)
         if map_coords:
             x, y = self(lons, lats)
         else:
@@ -546,50 +710,28 @@ class MapCanvas(basemap.Basemap):
         line segments. lons and lats are lists of waypoint coordinates.
         """
         # use great circle formula for a perfect sphere.
-        gc = pyproj.Geod(a=self.rmajor, b=self.rminor)
         assert len(lons) == len(lats)
         assert len(lons) > 1
         gclons = [lons[0]]
         gclats = [lats[0]]
         for i in range(len(lons) - 1):
-            az12, az21, dist = gc.inv(lons[i], lats[i], lons[i + 1], lats[i + 1])
+            _, _, dist = self.gc.inv(lons[i], lats[i], lons[i + 1], lats[i + 1])
             npoints = int((dist + 0.5 * 1000. * del_s) / (1000. * del_s))
-            # BUG -- weird path in cyl projection on waypoint move
-            # On some system configurations, the path is wrongly plotted when one
-            # of the waypoints is moved by the user and the current projection is cylindric.
-            # The weird thing is that when comparing cyl projection and stereo projection
-            # (which works), the exact same arguments are passed to gc.npts(). Also, the
-            # gc is initialised with the exact same a and b. Nevertheless, for the cyl
-            # projection, gc.npts() returns lons that connect lon1 and lat2, not lon1 and
-            # lon2 ... I cannot figure out why, maybe this is an issue in certain versions
-            # of pyproj?? (mr, 16Oct2012)
             lonlats = []
             if npoints > 0:
-                lonlats = gc.npts(lons[i], lats[i], lons[i + 1], lats[i + 1], npoints)
-            # The cylindrical projection of matplotlib is not periodic, that means that
-            # -170 longitude and 190 longitude are not identical. The gc projection however
-            # assumes identity and maps all longitudes to -180 to 180. This is no issue for
-            # most other projections.
-            # The clean solution would be to have a periodic display, where the locations
-            # and path are plotted periodically every 360 degree. As long as this is not
-            # supported by matplotlib.basemap, we "hack" this to map the path to the
-            # longitude range defined by the locations. This breaks potentially down in case
-            # that the locations are too far apart (>180 degree), but this is not the typical
-            # use case and will thus hopefully not pose a problem.
-            if self.projection == "cyl" and npoints > 0:
-                lonlats = np.asarray(lonlats)
-                milon = min(lons[i], lons[i + 1])
-                malon = max(lons[i], lons[i + 1])
-                sel = lonlats[:, 0] < milon
-                lonlats[sel, 0] += 360
-                sel = lonlats[:, 0] > malon
-                lonlats[sel, 0] -= 360
-
+                lonlats = self.gc.npts(lons[i], lats[i], lons[i + 1], lats[i + 1], npoints)
             for lon, lat in lonlats:
                 gclons.append(lon)
                 gclats.append(lat)
             gclons.append(lons[i + 1])
             gclats.append(lats[i + 1])
+        if self.projection == "cyl":  # hack for wraparound
+            lon_min, lon_max = self.llcrnrlon, self.urcrnrlon
+            gclons = np.array(gclons)
+            gclons[gclons < lon_min] += 360
+            gclons[gclons > lon_max] -= 360
+            idcs = np.where(abs(np.diff(gclons)) > 300)[0]
+            gclons[idcs] = np.nan
         if map_coords:
             x, y = self(gclons, gclats)
         else:
