@@ -506,19 +506,19 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         picker_default = config_loader(dataset="filepicker_default")
         self.add_plugin_submenu("CSV", "csv", load_from_csv, picker_default, plugin_type="Import")
         self.add_plugin_submenu("CSV", "csv", save_to_csv, picker_default, plugin_type="Export")
-        self.import_plugins = {"csv": load_from_csv}
-        self.export_plugins = {"csv": save_to_csv}
+        self.import_plugins = {"CSV": (load_from_csv, "csv")}
+        self.export_plugins = {"CSV": (save_to_csv, "csv")}
         self.add_import_plugins(picker_default)
         self.add_export_plugins(picker_default)
 
     def add_plugin_submenu(self, name, extension, function, pickertype, plugin_type="Import"):
         if plugin_type == "Import":
             menu = self.menuImportFlightTrack
-            action_name = "actionImportFlightTrack" + clean_string(extension)
+            action_name = "actionImportFlightTrack" + clean_string(name)
             handler = self.handle_import_local
         elif plugin_type == "Export":
             menu = self.menuExportActiveFlightTrack
-            action_name = "actionExportFlightTrack" + clean_string(extension)
+            action_name = "actionExportFlightTrack" + clean_string(name)
             handler = self.handle_export_local
 
         if hasattr(self, action_name):
@@ -555,7 +555,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
                     self, self.tr("file io plugin error import plugins"),
                     self.tr(f"ERROR: Configuration\n\n{self.import_plugins}\n\nthrows {type(ex)} error:\n{ex}"))
                 continue
-            self.import_plugins[extension] = imported_function
+            self.import_plugins[name] = (imported_function, extension)
 
     def add_export_plugins(self, picker_default):
         plugins = config_loader(dataset="export_plugins")
@@ -583,10 +583,10 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
                     self, self.tr("file io plugin error import plugins"),
                     self.tr(f"ERROR: Configuration\n\n{self.export_plugins}\n\nthrows {type(ex)} error:\n{ex}"))
                 continue
-            self.export_plugins[extension] = imported_function
+            self.export_plugins[name] = (imported_function, extension)
 
     def remove_plugins(self):
-        for name in self.import_plugins:
+        for name, _ in self.import_plugins.items():
             full_name = "actionImportFlightTrack" + clean_string(name)
             actions = [_x for _x in self.menuImportFlightTrack.actions()
                        if _x.objectName() == full_name]
@@ -595,7 +595,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
             delattr(self, full_name)
         self.import_plugins = {}
 
-        for name in self.export_plugins:
+        for name, _ in self.export_plugins.items():
             full_name = "actionExportFlightTrack" + clean_string(name)
             actions = [_x for _x in self.menuExportActiveFlightTrack.actions()
                        if _x.objectName() == full_name]
@@ -719,11 +719,7 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         self.mscolab.switch_to_local()
         # self.setWindowModality(QtCore.Qt.NonModal)
         self.active_flight_track = item.flighttrack_model
-        for i in range(self.listViews.count()):
-            view_item = self.listViews.item(i)
-            view_item.window.setFlightTrackModel(self.active_flight_track)
-            # local we have always all options enabled
-            view_item.window.enable_navbar_action_buttons()
+        self.update_active_flight_track()
         font = QtGui.QFont()
         for i in range(self.listFlightTracks.count()):
             self.listFlightTracks.item(i).setFont(font)
@@ -731,6 +727,16 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         item.setFont(font)
         self.menu_handler()
 
+    def update_active_flight_track(self, old_flight_track_name=None):
+        for i in range(self.listViews.count()):
+            view_item = self.listViews.item(i)
+            view_item.window.setFlightTrackModel(self.active_flight_track)
+            # local we have always all options enabled
+            view_item.window.enable_navbar_action_buttons()
+            if old_flight_track_name is not None:
+                view_item.window.setWindowTitle(view_item.window.windowTitle().replace(old_flight_track_name,
+                                                self.active_flight_track.name))
+
     def activate_selected_flight_track(self):
         item = self.listFlightTracks.currentItem()
         self.activate_flight_track(item)
@@ -756,35 +762,35 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
         """Slot for the 'Save Active Flight Track As' menu entry.
         """
         default_filename = os.path.join(self.last_save_directory, self.active_flight_track.name + ".ftml")
-        file_type = ["Flight track (*.ftml)"] + [f"Flight track (*.{ext})" for ext in self.export_plugins.keys()]
+        file_type = ["Flight track (*.ftml)"]
         filename = get_save_filename(
             self, "Save Flight Track", default_filename, ";;".join(file_type), pickertag="filepicker_default"
         )
         logging.debug("filename : '%s'", filename)
         if filename:
+            ext = "ftml"
             self.save_flight_track(filename)
+            self.active_flight_track.filename = filename
+            self.active_flight_track.name = fs.path.basename(filename.replace(f"{ext}", "").strip())
 
     def save_flight_track(self, file_name):
+        ext = ".ftml"
         if file_name:
-            if file_name.endswith('.ftml'):
+            if file_name.endswith(ext):
                 try:
                     self.active_flight_track.save_to_ftml(file_name)
                 except (OSError, IOError) as ex:
                     QtWidgets.QMessageBox.critical(
                         self, self.tr("Problem while saving flight track to FTML:"),
                         self.tr(f"ERROR: {type(ex)} {ex}"))
-            else:
-                ext = fs.path.splitext(file_name)[-1]
-                file_path = fs.path.basename(file_name)
-                _function = self.export_plugins[ext[1:]]
-                _function(file_name, file_path, self.active_flight_track.waypoints)
-                self.active_flight_track.filename = file_name
-                self.active_flight_track.name = fs.path.basename(file_name.replace(f"{ext}", "").strip())
 
             for idx in range(self.listFlightTracks.count()):
                 if self.listFlightTracks.item(idx).flighttrack_model == self.active_flight_track:
+                    old_filght_track_name = self.listFlightTracks.item(idx).text()
                     self.listFlightTracks.item(idx).setText(self.active_flight_track.name)
 
+            self.update_active_flight_track(old_filght_track_name)
+
     def close_selected_flight_track(self):
         """Slot to close the currently selected flight track. Flight tracks can
            only be closed if at least one other flight track remains open. The
@@ -889,10 +895,10 @@ class MSSMainWindow(QtWidgets.QMainWindow, ui.Ui_MSSMainWindow):
             self.listViews.item(0).window.handle_force_close()
         self.listViews.clear()
         self.remove_plugins()
-        self.add_plugins()
         if self.mscolab.token is not None:
             self.mscolab.logout()
         read_config_file()
+        self.add_plugins()
 
     def open_config_editor(self):
         """
